@@ -1,12 +1,11 @@
-use std::collections::HashMap;
-use std::rc::Rc;
-
 use crate::function::{BinaryFunction, Function, UnaryFunction};
 use crate::num::checked::CheckedNum;
 use crate::num::unchecked::UncheckedNum;
 use crate::ops::math::*;
 use crate::utils::ignore_case_str::eq_ignore_case;
 use crate::utils::ignore_case_string::IgnoreCaseString;
+use std::collections::{HashMap, HashSet};
+use std::rc::Rc;
 use validate::{OrPanic, TokenKind};
 
 /// Trait to provides the variables, constants and functions used for evaluate an expression.
@@ -174,7 +173,18 @@ impl<'a, N> DefaultContext<'a, N> {
         if self.functions.contains_key(&function_name) {
             panic!("A function named '{}' already exists", function_name);
         } else {
-            self.functions.insert(function_name, Rc::new(func));
+            let func = Rc::new(func);
+
+            if let Some(aliases) = func.aliases() {
+                for alias in aliases.iter().map(|s| IgnoreCaseString::from(*s)) {
+                    if self.functions.contains_key(&alias) {
+                        panic!("A function named '{}' already exists", alias);
+                    }
+
+                    self.functions.insert(alias, func.clone());
+                }
+            }
+            self.functions.insert(function_name, func);
         }
     }
 
@@ -191,7 +201,19 @@ impl<'a, N> DefaultContext<'a, N> {
         if self.unary_functions.contains_key(&function_name) {
             panic!("An unary function named '{}' already exists", function_name);
         } else {
-            self.unary_functions.insert(function_name, Rc::new(func));
+            let func = Rc::new(func);
+
+            if let Some(aliases) = func.aliases() {
+                for alias in aliases.iter().map(|s| IgnoreCaseString::from(*s)) {
+                    if self.unary_functions.contains_key(&alias) {
+                        panic!("An unary function named '{}' already exists", alias);
+                    }
+
+                    self.unary_functions.insert(alias, func.clone());
+                }
+            }
+
+            self.unary_functions.insert(function_name, func);
         }
     }
 
@@ -212,19 +234,25 @@ impl<'a, N> DefaultContext<'a, N> {
     #[inline]
     pub fn add_binary_function_as<F: BinaryFunction<N> + 'a>(&mut self, func: F, name: &str) {
         #[cfg(debug_assertions)]
-        {
-            if name.chars().count() == 1 {
-                validate::check_token_name(TokenKind::Operator, name).or_panic();
-            } else {
-                validate::check_token_name(TokenKind::Function, name).or_panic();
-            }
-        }
+        validate::check_token_name(TokenKind::Operator, name).or_panic();
 
         let function_name = IgnoreCaseString::from(name);
         if self.binary_functions.contains_key(&function_name) {
             panic!("A binary function named '{}' already exists", function_name);
         } else {
-            self.binary_functions.insert(function_name, Rc::new(func));
+            let func = Rc::new(func);
+
+            if let Some(aliases) = func.aliases() {
+                for alias in aliases.iter().map(|s| IgnoreCaseString::from(*s)) {
+                    if self.binary_functions.contains_key(&alias) {
+                        panic!("A binary function named '{}' already exists", alias);
+                    }
+
+                    self.binary_functions.insert(alias, func.clone());
+                }
+            }
+
+            self.binary_functions.insert(function_name, func);
         }
     }
 }
@@ -279,7 +307,11 @@ impl<'a, N> Context<'a, N> for DefaultContext<'a, N> {
         #[cfg(debug_assertions)]
         validate::check_token_name(TokenKind::Variable, name).or_panic();
 
-        if self.constants.keys().any(|k| eq_ignore_case(k.as_raw_str(), name)) {
+        if self
+            .constants
+            .keys()
+            .any(|k| eq_ignore_case(k.as_raw_str(), name))
+        {
             panic!("Invalid variable name, a constant named `{}` exists", name)
         } else {
             self.variables.insert(name.to_string(), value)
@@ -465,10 +497,10 @@ pub struct Config {
     pub implicit_mul: bool,
     /// Allows complex numbers.
     pub complex_number: bool,
-    /// Allows using custom grouping symbols for function calls, eg: `Max[1,2,3]`, `Sum<2,4,6>`
+    /// Allows using custom grouping symbols for function calls, eg: `Max[1,2,3]`, `Sum{2,4,6}`
     pub custom_function_call: bool,
     /// Stores the grouping symbols as: `(`, `)`, `[`, `]`.
-    grouping: HashMap<char, GroupingSymbol>,
+    grouping: HashSet<Grouping>,
 }
 
 impl Config {
@@ -476,7 +508,7 @@ impl Config {
     /// if is need an empty `Config` use `Default` instead.
     #[inline]
     pub fn new() -> Self {
-        Config::default().with_group_symbol('(', ')')
+        Config::default().with_grouping(Grouping::Parenthesis)
     }
 
     /// Enables implicit multiplication for this `Config`.
@@ -511,30 +543,17 @@ impl Config {
 
     /// Adds a pair of grouping symbols to this `Config`.
     ///
-    /// # Panics
-    /// If the config already contains the given symbol.
-    ///
     /// # Example
     /// ```
-    /// use prexel::context::Config;
+    /// use prexel::context::{Config, Grouping};
     ///
     /// // `Default` allows to create an empty config
     /// let mut config = Config::default()
-    ///     .with_group_symbol('(', ')')
-    ///     .with_group_symbol('[', ']');
+    ///     .with_grouping(Grouping::Parenthesis)
+    ///     .with_grouping(Grouping::Bracket);
     /// ```
-    pub fn with_group_symbol(mut self, open_group: char, close_group: char) -> Config {
-        let grouping = &mut self.grouping;
-        let grouping_symbol = GroupingSymbol::new(open_group, close_group);
-
-        if grouping.insert(open_group, grouping_symbol).is_some() {
-            panic!("Duplicated symbol: `{}`", open_group);
-        }
-
-        if grouping.insert(close_group, grouping_symbol).is_some() {
-            panic!("Duplicated symbol: `{}`", close_group);
-        }
-
+    pub fn with_grouping(mut self, grouping: Grouping) -> Config {
+        self.grouping.insert(grouping);
         self
     }
 
@@ -542,25 +561,25 @@ impl Config {
     ///
     /// # Examples
     /// ```
-    /// use prexel::context::Config;
+    /// use prexel::context::{Config, Grouping};
     ///
-    /// let mut config = Config::new().with_group_symbol('[', ']');
-    /// assert_eq!(('(', ')'), config.get_group_symbol('(').unwrap().as_tuple());
+    /// let mut config = Config::new().with_grouping(Grouping::Parenthesis);
+    /// assert_eq!(('(', ')'), config.get_group_symbol('(').unwrap());
     /// ```
     #[inline]
-    pub fn get_group_symbol(&self, symbol: char) -> Option<&GroupingSymbol> {
-        self.grouping.get(&symbol)
+    pub fn get_group_symbol(&self, symbol: char) -> Option<(char, char)> {
+        Grouping::new(symbol).map(|g| g.symbols())
     }
 
     /// Gets the grouping close for the specified grouping open.
     ///
     /// # Example
     /// ```
-    /// use prexel::context::Config;
+    /// use prexel::context::{Config, Grouping};
     ///
     /// let config = Config::default()
-    ///     .with_group_symbol('(', ')')
-    ///     .with_group_symbol('[', ']');
+    ///     .with_grouping(Grouping::Parenthesis)
+    ///     .with_grouping(Grouping::Bracket);
     ///
     /// assert_eq!(Some('('), config.get_group_open_for(')'));
     /// assert_eq!(Some('['), config.get_group_open_for(']'));
@@ -568,21 +587,20 @@ impl Config {
     /// ```
     #[inline]
     pub fn get_group_open_for(&self, group_close: char) -> Option<char> {
-        match self.get_group_symbol(group_close) {
-            Some(s) if s.group_close == group_close => Some(s.group_open),
-            _ => None,
-        }
+        Grouping::new(group_close)
+            .map(|g| g.open_for(group_close))
+            .flatten()
     }
 
     /// Gets the grouping close for the specified grouping open.
     ///
     /// # Example
     /// ```
-    /// use prexel::context::Config;
+    /// use prexel::context::{Config, Grouping};
     ///
     /// let config = Config::default()
-    ///     .with_group_symbol('(', ')')
-    ///     .with_group_symbol('[', ']');
+    ///     .with_grouping(Grouping::Parenthesis)
+    ///     .with_grouping(Grouping::Bracket);
     ///
     /// assert_eq!(Some(')'), config.get_group_close_for('('));
     /// assert_eq!(Some(']'), config.get_group_close_for('['));
@@ -590,53 +608,99 @@ impl Config {
     /// ```
     #[inline]
     pub fn get_group_close_for(&self, group_open: char) -> Option<char> {
-        match self.get_group_symbol(group_open) {
-            Some(s) if s.group_open == group_open => Some(s.group_close),
-            _ => None,
-        }
+        Grouping::new(group_open)
+            .map(|g| g.close_for(group_open))
+            .flatten()
     }
 
     /// Checks a value indicating if the given `char` is a group close symbol.
     #[inline]
     pub fn is_group_close(&self, group_close: char) -> bool {
-        self.grouping
-            .get(&group_close)
-            .map_or(false, |s| s.group_close == group_close)
+        Grouping::new(group_close)
+            .map(|g| g.is_close(group_close))
+            .unwrap_or(false)
     }
 
     /// Checks a value indicating if the given `char` is a group open symbol.
     #[inline]
     pub fn is_group_open(&self, group_open: char) -> bool {
-        self.grouping
-            .get(&group_open)
-            .map_or(false, |s| s.group_open == group_open)
+        Grouping::new(group_open)
+            .map(|g| g.is_open(group_open))
+            .unwrap_or(false)
     }
 }
 
-/// Represents a grouping symbol.
-#[derive(Debug, Copy, Clone, Eq, PartialEq, Hash)]
-pub struct GroupingSymbol {
-    /// The open symbol of teh grouping.
-    pub group_open: char,
-    /// The close symbol of teh grouping.
-    pub group_close: char,
+/// Represents a grouping symbol pair.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum Grouping {
+    /// Grouping using parentheses: `(` and `)`.
+    Parenthesis,
+    /// Grouping using square brackets: `[` and `]`.
+    Bracket,
+    /// Grouping using curly braces: `{` and `}`.
+    Brace,
 }
 
-impl GroupingSymbol {
-    /// Constructs a new `GroupingSymbol`.
-    #[inline]
-    pub fn new(group_open: char, group_close: char) -> Self {
-        assert_ne!(group_open, group_close);
-        GroupingSymbol {
-            group_open,
-            group_close,
+impl Grouping {
+    /// Constructs a `Grouping` from the given `char`s.
+    pub fn new(c: char) -> Option<Grouping> {
+        match c {
+            '(' | ')' => Some(Grouping::Parenthesis),
+            '[' | ']' => Some(Grouping::Bracket),
+            '{' | '}' => Some(Grouping::Brace),
+            _ => None,
         }
     }
 
-    /// Gets the open and close `char` symbols of this grouping.
     #[inline]
-    pub fn as_tuple(&self) -> (char, char) {
-        (self.group_open, self.group_close)
+    pub fn symbols(&self) -> (char, char) {
+        (self.open(), self.close())
+    }
+
+    pub fn open(&self) -> char {
+        match self {
+            Grouping::Parenthesis => '(',
+            Grouping::Bracket => '[',
+            Grouping::Brace => '{',
+        }
+    }
+
+    pub fn close(&self) -> char {
+        match self {
+            Grouping::Parenthesis => ')',
+            Grouping::Bracket => ']',
+            Grouping::Brace => '}',
+        }
+    }
+
+    /// Returns `true` if the given `char` is a grouping open symbol.
+    #[inline]
+    pub fn is_open(&self, c: char) -> bool {
+        self.open() == c
+    }
+
+    /// Returns `true` if the given `char` is a grouping close symbol.
+    #[inline]
+    pub fn is_close(&self, c: char) -> bool {
+        self.close() == c
+    }
+
+    /// Returns the grouping open symbol for the given `char`.
+    pub fn close_for(&self, c: char) -> Option<char> {
+        if self.is_open(c) {
+            Self::new(c).map(|g| g.close())
+        } else {
+            None
+        }
+    }
+
+    /// Returns the grouping close symbol for the given `char`.
+    pub fn open_for(&self, c: char) -> Option<char> {
+        if self.is_close(c) {
+            Self::new(c).map(|g| g.open())
+        } else {
+            None
+        }
     }
 }
 
@@ -784,25 +848,13 @@ mod tests {
     #[test]
     fn config_test() {
         let config = Config::default()
-            .with_group_symbol('(', ')')
-            .with_group_symbol('[', ']');
+            .with_grouping(Grouping::Parenthesis)
+            .with_grouping(Grouping::Bracket);
 
-        assert_eq!(
-            config.get_group_symbol('(').unwrap(),
-            &GroupingSymbol::new('(', ')')
-        );
-        assert_eq!(
-            config.get_group_symbol(')').unwrap(),
-            &GroupingSymbol::new('(', ')')
-        );
-        assert_eq!(
-            config.get_group_symbol('[').unwrap(),
-            &GroupingSymbol::new('[', ']')
-        );
-        assert_eq!(
-            config.get_group_symbol(']').unwrap(),
-            &GroupingSymbol::new('[', ']')
-        );
+        assert_eq!(config.get_group_symbol('(').unwrap(), ('(', ')'));
+        assert_eq!(config.get_group_symbol(')').unwrap(), ('(', ')'));
+        assert_eq!(config.get_group_symbol('[').unwrap(), ('[', ']'));
+        assert_eq!(config.get_group_symbol(']').unwrap(), ('[', ']'));
     }
 
     #[test]
@@ -824,4 +876,101 @@ mod tests {
         assert!(context.is_unary_function("λ"));
         assert!(context.is_function("f"));
     }
+
+    #[test]
+    fn function_aliases_test() {
+        struct SumFunction;
+        impl Function<f64> for SumFunction {
+            fn name(&self) -> &str {
+                "sum"
+            }
+
+            fn call(&self, args: &[f64]) -> Result<f64> {
+                Ok(args.iter().sum())
+            }
+
+            fn aliases(&self) -> Option<&[&str]> {
+                Some(&["add", "∑"])
+            }
+        }
+
+        let mut context: DefaultContext<f64> = DefaultContext::new();
+        context.add_function(SumFunction);
+
+        assert!(context.is_function("sum"));
+        assert!(context.is_function("add"));
+        assert!(context.is_function("∑"));
+
+        assert!(context.get_function("sum").is_some());
+        assert!(context.get_function("add").is_some());
+        assert!(context.get_function("∑").is_some());
+    }
+
+    #[test]
+    fn binary_function_alias_test() {
+        struct AddFunction;
+        impl BinaryFunction<f64> for AddFunction {
+            fn name(&self) -> &str {
+                "+"
+            }
+
+            fn aliases(&self) -> Option<&[&str]> {
+                Some(&["add", "plus"])
+            }
+
+            fn precedence(&self) -> Precedence {
+                Precedence::VERY_LOW
+            }
+
+            fn associativity(&self) -> Associativity {
+                Associativity::Left
+            }
+
+            fn call(&self, left: f64, right: f64) -> Result<f64> {
+                Ok(left + right)
+            }
+        }
+
+        let mut context: DefaultContext<f64> = DefaultContext::new();
+        context.add_binary_function(AddFunction);
+
+        assert!(context.is_binary_function("+"));
+        assert!(context.is_binary_function("add"));
+        assert!(context.is_binary_function("plus"));
+
+        assert!(context.get_binary_function("+").is_some());
+        assert!(context.get_binary_function("add").is_some());
+        assert!(context.get_binary_function("plus").is_some());
+    }
+
+    #[test]
+    fn unary_function_alias_test() {
+        struct NotFunction;
+        impl UnaryFunction<i64> for NotFunction {
+            fn name(&self) -> &str {
+                "not"
+            }
+
+            fn aliases(&self) -> Option<&[&str]> {
+                Some(&["¬"])
+            }
+
+            fn notation(&self) -> Notation {
+                Notation::Prefix
+            }
+
+            fn call(&self, arg: i64) -> Result<i64> {
+                Ok(!arg)
+            }
+        }
+
+        let mut context: DefaultContext<i64> = DefaultContext::new();
+        context.add_unary_function(NotFunction);
+
+        assert!(context.is_unary_function("not"));
+        assert!(context.is_unary_function("¬"));
+
+        assert!(context.get_unary_function("not").is_some());
+        assert!(context.get_unary_function("¬").is_some());
+    }
 }
