@@ -581,8 +581,8 @@ mod shunting_yard {
         while let Some(t) = operators.pop() {
             match t {
                 Token::GroupingOpen(c) => {
-                    if let Some(grouping) = context.config().get_group_symbol(c) {
-                        if grouping.group_close == group_close {
+                    if let Some((_, close)) = context.config().get_group_symbol(c) {
+                        if close == group_close {
                             is_group_open = true;
                             // If `arg_count` is not empty we are inside a function.
                             // So we pop the argument count and function token into the output stack.
@@ -897,11 +897,12 @@ mod shunting_yard {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::context::Config;
+    use crate::context::{Config, Grouping};
+    use crate::ops::math::Function;
 
     #[test]
     fn eval_test() {
-        let config = Config::new().with_group_symbol('[', ']');
+        let config = Config::new().with_grouping(Grouping::Parenthesis);
         let evaluator: Evaluator<i64> =
             Evaluator::with_context(DefaultContext::with_config_checked(config));
 
@@ -923,7 +924,7 @@ mod tests {
         assert!(evaluator.eval("((10)+(((2)))*(3))").is_ok());
         assert!(evaluator.eval("-(2)^(((4)))").is_ok());
         assert!(evaluator.eval("-(+(-(+(5))))").is_ok());
-        assert!(evaluator.eval("10--+2").is_ok());
+        assert!(evaluator.eval("10 - - +2").is_ok());
         assert!(evaluator.eval("+2!").is_ok());
         assert!(evaluator.eval("5 * Sin(40)").is_ok());
         assert!(evaluator.eval("Sin(30) * 5").is_ok());
@@ -1029,4 +1030,30 @@ mod tests {
 
         assert_eq!(evaluator.eval("x + 2").unwrap(), 12);
     }
+
+    #[test]
+    fn eval_with_alias_test() {
+        struct SumFunction;
+        impl Function<f64> for SumFunction {
+            fn name(&self) -> &str {
+                "sum"
+            }
+
+            fn call(&self, args: &[f64]) -> Result<f64> {
+                Ok(args.iter().sum())
+            }
+
+            fn aliases(&self) -> Option<&[&str]> {
+                Some(&["add", "∑"])
+            }
+        }
+
+        let mut context: DefaultContext<f64> = DefaultContext::new();
+        context.add_function(SumFunction);
+
+        let evaluator = Evaluator::with_context(context);
+        assert_eq!(evaluator.eval("sum(1, 2, 3, 4)").unwrap(), 10.0);
+        assert_eq!(evaluator.eval("add(1, 2, 3, 4)").unwrap(), 10.0);
+        assert_eq!(evaluator.eval("∑(1, 2, 3, 4)").unwrap(), 10.0);
+    }
 }
